@@ -69,10 +69,22 @@ int batchcount;		/* count of messages sent in current batch */
 flag peek_capable;	/* can we peek for better error recovery? */
 int mailserver_socket_temp = -1;	/* socket to free if connect timeout */ 
 
-static int timeoutcount;		/* count consecutive timeouts */
+volatile static int timeoutcount = 0;	/* count consecutive timeouts */
+volatile static int idletimeout = 0;	/* timeout occured in idle stage? */
 
 static jmp_buf	restart;
 
+int isidletimeout(void)
+/* last timeout occured in idle stage? */
+{
+    return idletimeout;
+}
+
+void resetidletimeout(void)
+{
+    idletimeout = 0;
+}
+
 void set_timeout(int timeleft)
 /* reset the nonresponse-timeout */
 {
@@ -92,8 +104,11 @@ void set_timeout(int timeleft)
 static RETSIGTYPE timeout_handler (int signal)
 /* handle SIGALRM signal indicating a server timeout */
 {
-    timeoutcount++;
-    longjmp(restart, THROW_TIMEOUT);
+    if(stage != STAGE_IDLE) {
+	timeoutcount++;
+	longjmp(restart, THROW_TIMEOUT);
+    } else
+	idletimeout = 1;
 }
 
 static RETSIGTYPE sigpipe_handler (int signal)
@@ -416,7 +431,10 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 	if (msgcodes[num-1] < 0)
 	{
 	    if ((msgcodes[num-1] == MSGLEN_TOOLARGE) && !check_only)
+	    {
 		mark_oversized(ctl, num, msgsizes[num-1]);
+		suppress_delete = TRUE;
+	    }
   		/* To avoid flooding the syslog when using --keep,
   		 * report "Skipped message" only when:
   		 *  1) --verbose is on, or
@@ -456,6 +474,7 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 	else
 	{
 	    flag wholesize = !ctl->server.base_protocol->fetch_body;
+	    flag separatefetchbody = (ctl->server.base_protocol->fetch_body) ? TRUE : FALSE;
 
 	    /* request a message */
 	    err = (ctl->server.base_protocol->fetch_headers)(mailserver_socket,ctl,num, &len);
@@ -497,45 +516,23 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 	     * output sink.  
 	     */
 	    err = readheaders(mailserver_socket, len, msgsizes[num-1],
-			     ctl, num);
+			     ctl, num,
+			     /* pass the suppress_readbody flag only if the underlying
+			      * protocol does not fetch the body separately */
+			     separatefetchbody ? 0 : &suppress_readbody);
 	    if (err == PS_RETAINED)
-	    {
 		suppress_forward = suppress_delete = retained = TRUE;
-		/* do not read the body only if the underlying protocol
-		 * allows the body to be fetched separately */
-		if (ctl->server.base_protocol->fetch_body)
-		    suppress_readbody = TRUE;
-	    }
 	    else if (err == PS_TRANSIENT)
-	    {
 		suppress_delete = suppress_forward = TRUE;
-		if (ctl->server.base_protocol->fetch_body)
-		    suppress_readbody = TRUE;
-	    }
 	    else if (err == PS_REFUSED)
-	    {
 		suppress_forward = TRUE;
-		if (ctl->server.base_protocol->fetch_body)
-		    suppress_readbody = TRUE;
-	    }
 	    else if (err == PS_TRUNCATED)
-	    {
-		if (ctl->server.base_protocol->fetch_body)
-		    suppress_readbody = TRUE;
-		len = 0;	/* suppress body processing */
-	    }
+		suppress_readbody = TRUE;
 	    else if (err)
 		return(err);
 
-	    /* 
-	     * If we're using IMAP4 or something else that
-	     * can fetch headers separately from bodies,
-	     * it's time to request the body now.  This
-	     * fetch may be skipped if we got an anti-spam
-	     * or other PS_REFUSED error response during
-	     * readheaders.
-	     */
-	    if (ctl->server.base_protocol->fetch_body && !suppress_readbody) 
+	    /* tell server we got it OK and resynchronize */
+	    if (separatefetchbody && ctl->server.base_protocol->trail)
 	    {
 		if (outlevel >= O_VERBOSE && !isafile(1))
 		{
@@ -545,9 +542,27 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 
 		if ((err = (ctl->server.base_protocol->trail)(mailserver_socket, ctl, num)))
 		    return(err);
-		len = 0;
-		if (!suppress_forward)
+	    }
+
+	    /* do not read the body which is not being forwarded only if
+	     * the underlying protocol allows the body to be fetched
+	     * separately */
+	    if (separatefetchbody && suppress_forward)
+		suppress_readbody = TRUE;
+
+	    /* 
+	     * If we're using IMAP4 or something else that
+	     * can fetch headers separately from bodies,
+	     * it's time to request the body now.  This
+	     * fetch may be skipped if we got an anti-spam
+	     * or other PS_REFUSED error response during
+	     * readheaders.
+	     */
+	    if (!suppress_readbody)
+	    {
+		if (separatefetchbody)
 		{
+		    len = -1;
 		    if ((err=(ctl->server.base_protocol->fetch_body)(mailserver_socket,ctl,num,&len)))
 			return(err);
 		    /*
@@ -563,22 +578,12 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 			report_complete(stdout,
 					GT_(" (%d body octets) "), len);
 		}
-	    }
 
-	    /* process the body now */
-	    if (len > 0)
-	    {
-		if (suppress_readbody)
-		{
-		    err = PS_SUCCESS;
-		}
-		else
-		{
-		    err = readbody(mailserver_socket,
-				  ctl,
-				  !suppress_forward,
-				  len);
-		}
+		/* process the body now */
+		err = readbody(mailserver_socket,
+			      ctl,
+			      !suppress_forward,
+			      len);
 		if (err == PS_TRANSIENT)
 		    suppress_delete = suppress_forward = TRUE;
 		else if (err)
@@ -663,7 +668,8 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 	}
 	else if (ctl->server.base_protocol->delete
 		 && !suppress_delete
-		 && ((msgcodes[num-1] >= 0) ? !ctl->keep : ctl->flush))
+		 && ((msgcodes[num-1] >= 0 && !ctl->keep)
+		     || (msgcodes[num-1] == MSGLEN_OLD && ctl->flush)))
 	{
 	    (*deletions)++;
 	    if (outlevel > O_SILENT) 
@@ -698,7 +704,7 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 	}
 
 	/* perhaps this as many as we're ready to handle */
-	if (maxfetch && maxfetch <= *fetches && *fetches < count)
+	if (maxfetch && maxfetch <= *fetches && num < count)
 	{
 	    report(stdout, GT_("fetchlimit %d reached; %d messages left on server %s account %s\n"),
 		   maxfetch, count - *fetches, ctl->server.truename, ctl->remotename);
@@ -908,8 +914,8 @@ const int maxfetch;		/* maximum number of messages to fetch */
 		if (namerec == (struct hostent *)NULL)
 		{
 		    report(stderr,
-			   GT_("couldn't find canonical DNS name of %s\n"),
-			   ctl->server.pollname);
+			   GT_("couldn't find canonical DNS name of %s (%s)\n"),
+			   ctl->server.pollname, ctl->server.queryname);
 		    err = PS_DNS;
 		    set_timeout(0);
 		    phase = oldphase;
@@ -1011,8 +1017,8 @@ const int maxfetch;		/* maximum number of messages to fetch */
 	}
 
 #ifdef SSL_ENABLE
-	/* Save the socket opened. Usefull if Fetchmail hangs on SSLOpen 
-	 * because the socket can be closed
+	/* Save the socket opened. Useful if Fetchmail hangs on SSLOpen 
+	 * because the socket can be closed.
 	 */
 	mailserver_socket_temp = mailserver_socket;
 	set_timeout(mytimeout);
@@ -1024,6 +1030,7 @@ const int maxfetch;		/* maximum number of messages to fetch */
 	    ctl->sslcertpath,ctl->sslfingerprint,realhost,ctl->server.pollname) == -1) 
 	{
 	    report(stderr, GT_("SSL connection failed.\n"));
+	    err = PS_AUTHFAIL;
 	    goto closeUp;
 	}
 	
@@ -1237,7 +1244,7 @@ is restored."));
 		    (void) sprintf(buf,
 #endif /* HAVE_SNPRINTF */
 				   GT_("%s at %s (folder %s)"),
-				   ctl->remotename, ctl->server.truename, idp->id);
+				   ctl->remotename, ctl->server.pollname, idp->id);
 		else
 #ifdef HAVE_SNPRINTF
 		    (void) snprintf(buf, sizeof(buf),
@@ -1245,7 +1252,7 @@ is restored."));
 		    (void) sprintf(buf,
 #endif /* HAVE_SNPRINTF */
 			       GT_("%s at %s"),
-				   ctl->remotename, ctl->server.truename);
+				   ctl->remotename, ctl->server.pollname);
 		if (outlevel > O_SILENT)
 		{
 		    if (count == -1)		/* only used for ETRN */
@@ -1253,10 +1260,12 @@ is restored."));
 		    else if (count != 0)
 		    {
 			if (new != -1 && (count - new) > 0)
-			    report_build(stdout, GT_("%d %s (%d seen) for %s"),
+			    report_build(stdout, GT_("%d %s (%d %s) for %s"),
 				  count, count > 1 ? GT_("messages") :
 				                     GT_("message"),
-				  count-new, buf);
+				  count-new, 
+				  GT_("seen"),
+				  buf);
 			else
 			    report_build(stdout, GT_("%d %s for %s"), 
 				  count, count > 1 ? GT_("messages") :
